@@ -43,17 +43,17 @@ query_usgs_geoserver <- function(AOI = NULL,  ids = NULL,
                                       'nhd','catchment', 'nhdarea',
                                       'nonnetwork',
                                       'waterbodies',
-                                      'gagesII'),
+                                      'gagesII', "gagesII-basin"),
                        geoserver  = c("huc08","huc12",
                                       "nhdflowline_network", "catchmentsp", 'nhdarea',
                                       "nhdflowline_nonnetwork",
                                       "nhdwaterbody",
-                                      "gagesii"),
+                                      "gagesii", "gagesii_basins"),
                        ids        = c("huc8", "huc12",
                                       "comid", "featureid", "comid",
                                       "comid",
                                       "comid",
-                                      "staid"))
+                                      "staid", "gage_id"))
 
   if(is.null(type)){ return(source) }
 
@@ -278,40 +278,3 @@ extact_comid_nwis <- function(nwis){
   f.comid  <-  jsonlite::fromJSON(c, simplifyVector = TRUE)
   f.comid$features$properties$comid
 }
-
-#' @title Discover NHDPlus ID
-#' @description Multipurpose function to find a COMID of interest.
-#' @param point An sf POINT including crs as created by:
-#' sf::st_sfc(sf::st_point(..,..), crs)
-#' @param nldi_feature list with names `featureSource` and `featureID` where
-#' `featureSource` is derived from the "source" column of  the response of
-#' \link[dataRetrieval]{get_nldi_sources} and the `featureSource` is a known identifier
-#' from the specified `featureSource`.
-#' @return integer COMID
-#' @export
-#' @examples
-#' \donttest{
-#' point <- sf::st_sfc(sf::st_point(c(-76.87479, 39.48233)), crs = 4326)
-#' discover_nhdplus_id(point)
-#'
-#' nldi_nwis <- list(featureSource = "nwissite", featureID = "USGS-08279500")
-#' discover_nhdplus_id(nldi_feature = nldi_nwis)
-#' }
-
-
-discover_nhdplus_id <- function(point = NULL, nldi_feature = NULL) {
-  if (!is.null(point)) {
-    coords = sf::st_coordinates(point)
-    comid  = dataRetrieval::findNLDI(location = c(X = coords[1],
-                                                  Y = coords[2]))$identifier
-    return(as.integer(comid))
-  } else if (!is.null(nldi_feature)) {
-
-    nldi <- get_nldi_feature(nldi_feature)
-
-    return(as.integer(nldi$comid))
-
-  } else {
-    stop("Must provide point or nldi_feature input.")
-  }
-}
