@@ -346,3 +346,265 @@ get_start_comid <- function(network, comid) {
 
   start_comid
 }
+
+#' Navigate Network
+#' @description Provides a full feature network navigation function that
+#' will work with local or web service data. Parameter details provide
+#' context.
+#' @param start list, integer, sf, or sfc if list must be a valid NLDI feature
+#' if integer must be a valid comid. If sf, must contain a "comid" field.
+#' @param mode character chosen from c(UM, DM, UT, or DD)
+#' @param network sf should be compatible with network navigation functions
+#' If NULL, network will be derived from requests to the NLDI
+#' @param output character flowline or a valid NLDI data source
+#' @param distance_km numeric distance to navigate in km
+#' @param trim_start logical should start be trimmed or include entire catchment?
+#' @param trim_stop logical should stop(s) be trimmed or include entire catchment(s)?
+#' # Not supported
+#' @param trim_tolerance numeric from 0 to 100 percent of flowline length. If amount
+#' to trim is less than this tolerance, no trim will be applied.
+#' @export
+#' @examples
+#'
+#' \donttest{
+#' navigate_network(list(featureSource = "nwissite", featureID = "USGS-06287800"),
+#'                 "UM",
+#'                 output = "flowlines",
+#'                 trim_start = TRUE)
+#' }
+#'
+#' source(system.file("extdata", "walker_data.R", package = "nhdplusTools"))
+#' hydro_location <- list(comid = 5329339,
+#'                       reachcode = "18050005000078",
+#'                        reach_meas = 30)
+#'
+#' hydro_location <- sf::st_sf(
+#'   hydro_location,
+#'   geom = nhdplusTools::get_hydro_location(data.frame(hydro_location),
+#'                                           walker_flowline))
+#'
+#' net <- navigate_network(hydro_location,
+#'                        mode = "DM", network = walker_flowline,
+#'                        trim_start = TRUE, distance_km = 20)
+#'
+#' plot(sf::st_geometry(walker_flowline))
+#' plot(sf::st_geometry(hydro_location), add = TRUE)
+#' plot(sf::st_geometry(net), add = TRUE, col = "blue", lwd = 2)
+#'
+navigate_network <- function(start, mode = "UM", network = NULL,
+                             output = "flowlines", distance_km = 10,
+                             trim_start = FALSE, trim_stop = FALSE,
+                             trim_tolerance = 5) {
+
+  # Work with start to get a start_comid
+  if(!is.numeric(start)) {
+    if(inherits(start, "sfc")) {
+
+      start_comid <- discover_nhdplus_id(point = start)
+
+    } else if(inherits(start, "sf")) {
+
+      start_comid <- start$comid
+
+    } else if(is.list(start)) {
+
+      start <- get_nldi_feature(start)
+      start_comid <- as.integer(start$comid)
+
+    } else {
+
+      stop("start must be integer, list, or sfc geometry")
+
+    }
+
+  } else {
+
+    if(!start %% 1 == 0) {
+      stop("If start is numeric it must be a comid integer")
+    }
+
+    start <- as.integer(start)
+    start_comid <- start
+
+  }
+
+  # in the case that we don't have a network, we need to get it
+  # from the NLDI and web services.
+  if(is.null(network)) {
+
+    network <- navigate_nldi(list(featureSource = "comid",
+                                  featureID = start_comid),
+                             mode, "flowlines", distance_km)
+
+    network <- network[names(network) != "origin"][[1]]$nhdplus_comid
+
+    network <- subset_nhdplus(as.integer(network),
+                              nhdplus_data = "download",
+                              status = TRUE,
+                              flowline_only = TRUE)
+
+    network <- network$NHDFlowline_Network
+
+  } else {
+
+    # If we have a network, we need to filter it down to
+    # the desired navigation.
+    network <- check_names(network, paste0("get_", mode),
+                           tolower = TRUE)
+
+    if(!start_comid %in% network$comid)
+      stop("start comid not in network?")
+
+    network <- dplyr::filter(network, .data$comid %in%
+                               if(mode == "UM") {
+                                 get_UM(network, start_comid, distance_km, include = TRUE)
+                               } else if(mode == "UT") {
+                                 get_UT(network, start_comid, distance_km)
+                               } else if(mode == "DD") {
+                                 get_DD(network, start_comid, distance_km)
+                               } else if(mode == "DM") {
+                                 get_DM(network, start_comid, distance_km, include = TRUE)
+                               }
+    )
+
+  }
+
+
+  # we now have a network that matches our navigation and, if trim options are
+  # FALSE, can output the desired output data.
+
+  # we need a precise measure for our point
+  if(trim_start) {
+
+    if(!is.numeric(start)) {
+
+      if(inherits(start, "sf") | inherits(start, "sfc")) {
+        start <- sf::st_transform(start,
+                                  sf::st_crs(network))
+      }
+
+      if(sf::st_is_longlat(start)) {
+        search_radius <- 0.001
+      } else {
+        search_radius <- 100
+      }
+
+      event <- get_flowline_index(flines = network,
+                                 points = start,
+                                 search_radius = search_radius,
+                                 precision = 10)
+
+      event <- sf::st_sf(event,
+                         geom = get_hydro_location(event, network))
+
+    }
+
+  }
+
+  # get the output data if it's not flowlines
+  if(!output == "flowlines") {
+
+    out_features <- navigate_nldi(list(featureSource = "comid",
+                                       featureID = start_comid),
+                                  mode, output, distance_km
+                                  )[[paste0(mode, "_", output)]]
+
+  } else {
+
+    out_features <- NULL
+
+  }
+
+  if(trim_stop) {
+
+    stop("Trim Stop Not Supported")
+
+  }
+
+  # now trim start if requested
+  if(trim_start) {
+
+    if(!is.integer(start)) {
+      if(output == "flowlines") {
+        # trim event flowline to measure of event
+
+        l <- network[network$comid == start_comid, ]
+
+        rm <- as.numeric(event$REACH_meas)
+        lf <- l$frommeas
+        lt <- l$tomeas
+
+        # the event measure can be outside the network?
+        if(!dplyr::between(rm, lf, lt)) {
+          df <- abs(c(lf, lt) - rm)
+          split <- c(lf, lt)[which(df == min(df))]
+        } else {
+          # Convert start to comid measure
+          split <- rescale_measures(as.numeric(event$REACH_meas),
+                                    l$frommeas, l$tomeas)
+        }
+
+        if(grepl("UM|UT", mode)) {
+
+          f <- 0
+          t <- 1 - (split / 100)
+
+        } else {
+
+          f <- 1 - (split / 100)
+          t <- 1
+
+        }
+
+        if(abs(f - t) < trim_tolerance / 100) {
+
+          warning("No split applied, under tolerance.")
+
+        } else {
+
+          sf::st_geometry(network)[network$comid == start_comid] <-
+            suppressWarnings({
+
+              if(!requireNamespace("lwgeom")) {
+                stop("lwgeom required to trim flowlines to a specific measure.")
+              }
+
+              lwgeom::st_linesubstring(
+                sf::st_cast(sf::st_geometry(l), "LINESTRING"), f, t)
+
+            })
+        }
+
+      } else {
+        if("measure" %in% names(out_features)) {
+          # remove if reachcode is same as start and
+          # measure is greater for upstream
+          # measure is less for downstream
+
+          out_features <- dplyr::filter(
+            out_features,
+            if(grepl("UM|UT", mode)) {
+              .data$reachcode != event$REACHCODE |
+                (.data$reachcode == event$REACHCODE & .data$measure > event$REACH_meas)
+            } else {
+              .data$reachcode != event$REACHCODE |
+                (.data$reachcode == event$REACHCODE & .data$measure < event$REACH_meas)
+            })
+
+        }
+      }
+    } else {
+
+      warning("trim_start ignored for comid start")
+
+    }
+
+  }
+
+  if(!is.null(out_features)) {
+    return(out_features)
+  } else {
+    return(network)
+  }
+
+}
