@@ -1,8 +1,10 @@
-context("run plus attributes")
+
 
 test_that("example", {
   source(system.file("extdata", "walker_data.R", package = "nhdplusTools"))
 
+  expect_s3_class(get_sorted(walker_flowline), "sf")
+
   test_flowline <- prepare_nhdplus(walker_flowline, 0, 0, FALSE, warn = FALSE)
 
   test_flowline <- data.frame(
@@ -31,12 +33,29 @@ test_that("example", {
                                      split_temp = (tempf <- tempfile(fileext = ".rds")),
                                      status = FALSE)
 
-  expect_equivalent(fl, fl2)
+  expect_equal(fl, fl2, ignore_attr = TRUE)
 
   expect_true(file.exists(tempf))
 
-  expect_is(readRDS(tempf), "list")
+  expect_type(readRDS(tempf), "list")
 
   unlink(tempf)
 })
 
+test_that("coastal", {
+  net <- read.csv(list.files(pattern = "coastal_net.csv", full.names = TRUE, recursive = TRUE))
+
+  net <- net[order(net$hydroseq), ]
+
+  sorted <- get_sorted(net[, c("comid", "tocomid")], split = TRUE)
+
+  expect_true(which(sorted$comid == 2544325) <
+                which(sorted$comid == 2544321))
+
+  expect_equal(length(unique(sorted$terminalID)), 32)
+
+  sorted <- get_sorted(net[, c("comid", "tocomid")], split = TRUE, outlets = 2544457)
+
+  expect_true(all(sorted$terminalID == 2544457))
+
+})
